/* Copyright (C) 1995  Sam Moore, Warren Jasper
 * Copyright (C) 1995  Ricky Beam, Rob Cox
 * Copyright (C) 1996  Joerg Weule
 * Copyright (C) 1998  Jim Bean
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/** adclib.c **/

#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include "adc.h"

/* Constants for count-to-voltage conversion routine. */
static const float Gains_1402 [4] = {10.0, 5.0, 2.5, 1.25};
static const float Gains_1401 [4] = {10.0, 1.0, 0.1, 0.01};

/* Factors for count-to-voltage conversion routine. */
static float Gain       = 5.0;
static float Subtractor = 0.0;
static float Divisor    = 65535.0;

static int Bits_12      = 0; /* 12 bit or 16 bit conversion selectable. */
static int Chans        = 1;
static LONG Unipolar    = 1;

static char *DevName    = "/dev/adc00";

/********************************************************************
 * The 12 bit machine returns the channel number as 4 bits of the 16 bit
 * return.  Extract the 4 bits and make a proper 12 bit number form the rest.
 *
 * For the 16 bit machine the channel number is
 * the point number modulo the number of channels.
 ********************************************************************/

void ConvertData (long NumPoints, WORD *ADData, int *ChanTags) {
  
  int i;
  
  for (i=0; i < NumPoints; i++) {
    if (Bits_12) {
      ChanTags[i] = (ADData[i] & 0xf);
      ADData[i] >>= 4;
    }
    else ChanTags[i] = i % Chans;
  }
}

/****************************************************************
 * Convert the raw count to a float using the supplied factors.
 * Board type       (1401, 1402)
 * Number of bits   (12 bits, 16 bits)
 * Gain Selected    (4 values according to board type)
 * Unipolar or Bipolar
 ****************************************************************/

void RawToReal (long NumPoints, WORD *RawData, float *RealData) {

  int i;

  for (i=0; i < NumPoints; i++) {
    RealData[i] = (RawData[i] - Subtractor) * (Gain / Divisor);
  }
}

/**************************************************************
 *  Local open used by all board specific opens.
 **************************************************************/

__inline int adcopen (int Mode, int Freq, int Gain_Sel, int Ch, int DMALen) {

  int	fd;

  if ((fd = open(DevName, Mode )) < 0) {
    perror(DevName);
    printf("error opening device %s\n", DevName);
    exit(2);
  }

  /* Set voltage gain */

  printf("Setting input range to %d. Look at manual.\n", Gain_Sel);
  ioctl(fd, ADC_SET_GAINS, Gain_Sel);

  /* program counter 0 */
  ioctl(fd, COUNTER0, CTR0_MODE2 | 10) ;
  ioctl(fd, CTR0, 0x1);

  Chans = Ch;
  ioctl(fd, ADC_SETMUX_LIMITS, Ch-1);

  if ( Mode == ADC_SOFT_TRIGGER ){
       ioctl(fd, ADC_SET_DMA_LEN, (void*)Chans);
       ioctl(fd, ADC_START_SOFT_TRIG , 0 );
  }

  if ( Mode == ADC_PACER_CLOCK ){
       ioctl(fd, ADC_SET_DMA_LEN, (void*)DMALen);

       /* Set sampling frequency */
       printf("Requested frequency at %d Hz.\n", Freq);
       ioctl(fd, ADC_SET_PACER_FREQ, Freq);
       ioctl(fd, ADC_GET_PACER_FREQ, &Freq);
       printf("Actual frequency set to %d Hz.\n", Freq);

       ioctl(fd, ADC_START_PACER , 0 );
  }

  /* Setup for the Raw to Real conversion */

  /* Unipolar is set by a switch on the board, get the setting. */
  ioctl(fd, ADC_GET_RANGE, &Unipolar);

  if (Unipolar){
     Subtractor = 0.0;
     if (Bits_12) Divisor =  4095.0;
     else         Divisor = 65535.0;
  }
  else {
     if (Bits_12){
        Subtractor =  2048.0;
        Divisor =     2047.0;
     }
     else {
        Subtractor = 32768.0;
        Divisor =    32767.0;
     }
  }

  return fd;

} /* adcopen */

/***********************************
 * open for the das1402 board.
 **********************************/

int adcopen_1402_16 (int Mode, int Freq, int Gain_Sel, int Ch, int DMALen) {

  int fd;

  Bits_12 = 0;
  fd = adcopen (Mode, Freq, Gain_Sel, Ch, DMALen);

  Gain = Gains_1402 [Gain_Sel];

  return fd;

} /* 1402/16 */

int adcopen_1402_12 (int Mode, int Freq, int Gain_Sel, int Ch, int DMALen) {

  int fd;

  Bits_12 = 1;
  fd = adcopen (Mode, Freq, Gain_Sel, Ch, DMALen);

  Gain = Gains_1402 [Gain_Sel];

  return fd;

} /* 1402/12 */

/***********************************
 * open for the das1401 board.
 **********************************/

int adcopen_1401_12 (int Mode, int Freq, int Gain_Sel, int Ch, int DMALen){

  int	fd;

  Bits_12 = 1;
  fd = adcopen (Mode, Freq, Gain_Sel, Ch, DMALen);

  Gain = Gains_1401 [Gain_Sel];

  return fd;

} /* 1401/12 */

/***********************************
 * read Analog data.
 **********************************/

int adcread (int fd, WORD *buf, int Datasets) {

  int Bytes;

  Bytes = read (fd, buf, Datasets);

  return Bytes / 2;

}

/***********************************
 * read Digital data.
 **********************************/

int DIOread (int fd) {

  int FourBits;

  ioctl(fd, ADC_DIO_IN, &FourBits);

  return FourBits;
}

/***********************************
 * write Digital data.
 **********************************/

void DIOwrite (int fd, int Val) {

  ioctl(fd, ADC_DIO_OUT, Val);

}

/***********************************
 * close.
 **********************************/

void adclose (int fd) {

  close (fd);

}
