/*
 * Copyright (C) 1995  Sam Moore, Warren Jasper  (CIO-DAS16/330)
 * Copyright (C) 1995  Ricky Beam, Rob Cox       (CIO-DAS1400)
 * All rights reserved.
 *
 * This software may be freely copied, modified, and redistributed
 * provided that this copyright notice is preserved on all copies.
 *
 * You may not distribute this software, in whole or in part, as part of
 * any commercial product without the express consent of the authors.
 *
 * There is no warranty or other guarantee of fitness of this software
 * for any purpose.  It is provided solely "as is".
 *
 */

/*   das1400.h    */

#define KERNEL_VERSION_2_0

/* Default device major number */
#ifndef DEFAULT_MAJOR_DEV
#define DEFAULT_MAJOR_DEV  30    /* Default Major Device Number */
#endif
#define DEFAULT_FREQ        50   /* Default frequency */
#define DAS_IRQ              7   /* Interrupt for read() 2-7 */
#define DAS_DMA              1   /* DMA Channel  */
#define AD_CHANNELS         16   /* 16 Single Ended or 8 Diff. */
#define DIO_CHANNELS         4   /* 4 Input and 4 Output */
#define BASE_REG         0x300   /* base address for CIO-DAS 1400 */

/* I/O Register locations */

#define LSB_AND_CHNLS      BASE_REG      /* A/D Data & Channel Register */
#define MSB_DATA_BYTE      BASE_REG+1    
#define MUX_SCAN_LIMITS    BASE_REG+2    /* Channel Mux Scan Limits Register */
#define DIO_REG            BASE_REG+3    /* 4 Bit Digital I/O Register */
#define STATUS_REG         BASE_REG+8    /* Status Register */
#define CNTRL_REG          BASE_REG+9    /* DMA, Interrupt & Trigger Control */
#define PACER_CLOCK_REG    BASE_REG+10   /* Pacer Clock Control Register */
#define MISC_REG           BASE_REG+11   /* Analog Input Range Reg., ETC */

#define COUNTERA_0_DATA    BASE_REG+12   /* Pacer Clock A Counter 0 Register */
#define COUNTERA_1_DATA    BASE_REG+13   /* Pacer Clock A Counter 1 Register */
#define COUNTERA_2_DATA    BASE_REG+14   /* Pacer Clock A Counter 2 Register */
#define COUNTERA_CONTROL   BASE_REG+15   /* Pacer Clock A Control  Register */

#define CONVERT_DIS_REG    BASE_REG+0x404  /* Prevent A/D trigger conversions */
#define BURST_MODE_REG     BASE_REG+0x405  /* DAS1400 Burst Mode Enable/Disable */
#define MODE_ENABLE_REG    BASE_REG+0x406  /* DAS1400 functions Enable/Disable */
#define BURST_STAT_REG     BASE_REG+0x407  /* DAS1400 Board Status Register    */

#define DAS1400_BME 1<<6
#define DAS1400_ME  1<<5
#define DAS1400_CD  1<<4
#define DAS1400_WS  1<<1
#define DAS1400_CLK 1<<0

#define DA_MINOR           24
#define DA_PORT0           BASE_REG+4

/* Errors */

#define BAD_CHANNEL        1
#define BAD_COUNT          2
#define BAD_GAIN           3
#define NO_ENHANCED_MODE   4
#define BAD_SPEED          5
#define ADC_BUSY           6
#define INT_ALREADY_ACTIVE 7

/* General defines */

#define TRUE  1
#define FALSE 0

/* #define DEBUG */

/* Interrupt context data. Controls interrupt service handler */

typedef struct ADC_REC {
  LONG freq;           /* Complete frequency value */
  WORD ctr1;           /* MSB of estimated "freq" */
  WORD ctr2;           /* LSB of estimated "freq" */
  BYTE pacerReg;       /* value of pacer clock control register */
} BoardRec;

/* Values kept for each channel. */

typedef struct CHANNEL_REC {
  int  open;            /* Is channel device open() */
  int  pacer;           /* Pacer and DMA is active */
  int  adc_dio_pretrig; /* TRUE = write dio before sampling                  */
  BYTE dio_value;       /* DIO value to be written before sampling           */
  LONG total;           /* Number of samples completed                       */
  BYTE mode;            /* 0 = Compatibility Mode, Soft Trigger              */
                        /* 2 = Extended Mode,      External Trigger          */
                        /* 4 = Extended Mode,      Pacer Clock               */
                        /* 6 = Extended Mode,      Pacer Clock               */
                        /*                           External Trig Enable    */
  BYTE gain;            /* Voltage range */
  BYTE Unipolar;        /* BiPolar = 0 or Unipolar = 1                       */
  BYTE Is_16_Ch;        /* 8 Ch diff = 0 or 16 Ch SE = 1                     */
  BYTE highScan,
       lowScan;         /* Mux Scan limits (expt.)                           */
  int  nScan;           /* highScan - lowScan + 1                            */
} ChanRec;


/*************************************************************************
* STATUS_PORT        BASE_REG+8             Status Register              *
**************************************************************************/
#define ADSR_INT        0x10      /* INT=1 External pulse received, INT=0 waiting */
#define ADSR_MUX        0x20      /* MUX=1 16 Single Ended, MUX=0 8 Differential */
#define ADSR_UB         0x40      /* U/B=1 Unipolar mode, U/B=0 Bipolar mode */
#define ADSR_EOC        0x80      /* EOC=1, A/D is busy, EOC=0 free */

/*************************************************************************
* CNTRL_REG          BASE_REG+9        DMA, Interrupt & Trigger Control  *
**************************************************************************/
#define CNTL_TS0       0x1        /* TS0=0 External Trigger, TS0=1 Pacer Clock */
#define CNTL_TS1       0x2        /* TS1=0 Software triggered A/D only */
#define CNTL_DMA       0x4        /* DMA Disabled = 0, DMA Enabled = 1 */
#define CNTL_INTE      0x80       /* Interrupts Disabled = 0, Enabled = 1 */

/*************************************************************************
* MISC_REG        BASE_REG+11         Range, and Exteded Features        *
**************************************************************************/

#define ENHANCED_BIT  (0x10)     /* Enhanced Mode Enabled = 1, Disabled = 0 */
#define MISC_OVERRUN  (0x20)     /* No overrun = 0, FIFO buffer full = 1 */
#define MISC_PRETRIG  (0x40)     /* Pretrigger Enabled = 1, Disable = 0 */
#define MISC_DT       (0x80)     /* DT-Connect Enable = 1, Disable = 0 */

/*************************************************************************
* Constants for dealing with the 8254 counters                           *
**************************************************************************/

#define MODE0 0x0
#define MODE1 0x2
#define MODE2 0x4
#define MODE3 0x6
#define MODE4 0x8
#define MODE5 0xa

#define C0 0x00
#define C1 0x40
#define C2 0x80

#define CTRLATCH 0x00
#define LSBONLY  0x10
#define MSBONLY  0x20
#define LSBFIRST 0x30

#define S0           0x00
#define S1           0x02

